@@ -38,7 +38,7 @@ use constant ETHER2TOKEN => (
 
 use base qw( Exporter );
 use vars qw( $VERSION %EXPORT_TAGS @EXPORT_OK );
-$VERSION = (qw$Revision: 0.79 $)[1];
+$VERSION = (qw$Revision: 0.80 $)[1];
 
 %EXPORT_TAGS = (
     all => [
@@ -183,10 +183,26 @@ Feel free to send patches for features you add.
 
 Creates and returns a new NetAddr::MAC object.  The MAC value is required.
 
+=head2 NetAddr::MAC->new( mac => $mac, %options )
+
+As above, but %options may include any or none of the following
+
+=over 4
+
+=item * die_on_error
+
+If set to true, errors will result in a die (croak) rather than populating $errstr
+
+=back
+
 =head2 NetAddr::MAC->new( $mac )
 
 Simplified creation method
 
+=head2 NetAddr::MAC->new( $mac, %options )
+
+As above but with %options
+
 =cut
 
 sub new {
@@ -195,71 +211,123 @@ sub new {
     my $c = ref($p) || $p;
     my $self = bless {}, $c;
 
-    croak q|Please provide a mac address|
-      unless @a;
+    unless (@a) {
+        my $e = q|Please provide a mac address|;
+        croak "$e\n" if $NetAddr::MAC::die_on_error;
+        $NetAddr::MAC::errstr = $e;
+        return;
+    }
 
     # massage a single argument into a mac argument if needed
-    $self->_init( @a == 1 ? ( mac => shift @a ) : @a );
+    $self->_init( @a % 2 ? ( mac => shift @a, @a ) : @a )
+      or return;
 
     return $self;
 
 }
 
-sub _init {
+{
 
-    my ( $self, %args ) = @_;
+    my $_die;
 
-    $self->{mac}      = _mac_to_integers( $args{mac} );
-    $self->{original} = $args{mac};
+    sub _init {
 
-    return;
+        my ( $self, %args ) = @_;
+
+        if ( defined $args{die_on_error} ) {
+            $self->{_die}++ if $args{die_on_error};
+        }
+        else {
+            $self->{_die}++ if $NetAddr::MAC::die_on_error;
+        }
+
+        $_die++ if $self->{_die};
+
+        $self->{original} = $args{mac};
+
+        $self->{mac} = _mac_to_integers( $args{mac} );
+
+        unless ( $self->{mac} ) {
+            croak $NetAddr::MAC::errstr . "\n" if $self->{_die};
+            return;
+        }
+
+        return 1;
+
+    }
+
+    sub _mac_to_integers {
+
+        my $mac = shift;
+        my $e;
+
+        for (1) {
+
+            unless ($mac) {
+                $e = 'Please provide a mac address';
+                last;
+            }
+
+            # be nice, strip leading and trailing whitespace
+            $mac =~ s/^\s+//;
+            $mac =~ s/\s+$//;
+
+            $mac =~ s{^1,\d,}{}
+              ; # blindly remove the prefix from bpr, we could check that \d is the actual length, but oh well
+
+            my @parts = grep { length } split( /[^a-z0-9]+/ix, $mac );
+
+            # anything other than hex...
+            last if ( first { m{[^a-f0-9]}i } @parts );
+
+            # 12 characters for EUI-48, 16 for EUI-64
+            if (
+                @parts == 1
+                && (   length $parts[0] == EUI48LENGTHHEX
+                    || length $parts[0] == EUI64LENGTHHEX )
+              )
+            {    # 0019e3010e72
+                local $_ = shift(@parts);
+                while (m{([a-f0-9]{2})}igx) { push( @parts, $1 ) }
+                return [ map { hex($_) } @parts ];
+            }
+
+            # 00:19:e3:01:0e:72
+            if ( @parts == EUI48LENGTHDEC || @parts == EUI64LENGTHDEC ) {
+                return [ map { hex($_) } @parts ];
+            }
+
+            # 0019:e301:0e72
+            if ( @parts == EUI48LENGTHDEC / 2 || @parts == EUI64LENGTHDEC / 2 )
+            {
+                return [
+                    map {
+                        m{^ ([a-f0-9]{2}) ([a-f0-9]{2}) $}ix
+                          && ( hex($1), hex($2) )
+                    } @parts
+                ];
+            }
+
+            last
+
+        } # just so we can jump out
+
+        $e ||= "Invalid MAC format '$mac'";
+
+        if ( defined $_die ) {
+            croak "$e\n" if $_die;
+        }
+        elsif ($NetAddr::MAC::die_on_error) {
+            croak "$e\n";
+        }
+
+        $NetAddr::MAC::errstr = $e;
+
+        return;
+    }
 
 }
 
-sub _mac_to_integers {
-
-    my $mac = shift || return;
-
-    $mac =~ s{^1,\d,}{}
-      ; # blindly remove the prefix from bpr, we could check that \d is the actual length, but oh well
-
-    my @parts = grep { length } split( /[^a-z0-9]+/ix, $mac );
-
-    croak "Invalid MAC format '$mac'"
-      if first { m{[^a-f0-9]}i } @parts;
-
-    # 12 characters for EUI-48, 16 for EUI-64
-    if (
-        @parts == 1
-        && (   length $parts[0] == EUI48LENGTHHEX
-            || length $parts[0] == EUI64LENGTHHEX )
-      )
-    {    # 0019e3010e72
-        local $_ = shift(@parts);
-        while (m{([a-f0-9]{2})}igx) { push( @parts, $1 ) }
-        return [ map { hex($_) } @parts ];
-    }
-
-    # 00:19:e3:01:0e:72
-    if ( @parts == EUI48LENGTHDEC || @parts == EUI64LENGTHDEC ) {
-        return [ map { hex($_) } @parts ];
-    }
-
-    # 0019:e301:0e72
-    if ( @parts == EUI48LENGTHDEC / 2 || @parts == EUI64LENGTHDEC / 2 ) {
-        return [
-            map { m{^ ([a-f0-9]{2}) ([a-f0-9]{2}) $}ix && ( hex($1), hex($2) ) }
-              @parts
-        ];
-    }
-
-    croak "Invalid MAC format '$mac'";
-
-    return;
-}
-
-=head1 OO METHODS
-
 =head2 original
 
 returns the original B<mac> string as used when creating the MAC object
@@ -273,6 +341,20 @@ sub original {
 
 }
 
+=head2 errstr
+
+returns the error (if one occured)
+
+=cut
+
+sub errstr {
+
+    my $self = shift;
+    return $NetAddr::MAC::errstr unless ref $self;
+    return $self->{_errstr}
+
+}
+
 =head1 OO PROPERTY METHODS
 
 =head2 is_eui48
@@ -436,7 +518,7 @@ sub as_ipv6_suffix {
             my $i = $_;
             $i *= 2;
             sprintf( '%02x%02x', $suffix[$i], $suffix[ $i + 1 ] )
-          } 0 .. 3
+        } 0 .. 3
     );
 }
 
@@ -464,16 +546,17 @@ returns the mac address normalized as a hexidecimal string that is 0 padded and
 
 sub as_singledash {
     my $self = shift;
+
     # there may be a better way to do this
-    my $len  = scalar @{ $self->{mac} };
+    my $len = scalar @{ $self->{mac} };
     return join(
         q{-},
         join( '',
             map { sprintf( '%02x', $_ ) }
-              @{ $self->{mac} }[ 0 .. ($len / 2 - 1) ] ),
+              @{ $self->{mac} }[ 0 .. ( $len / 2 - 1 ) ] ),
         join( '',
             map { sprintf( '%02x', $_ ) }
-              @{ $self->{mac} }[ ($len / 2)  .. ($len - 1) ] ),
+              @{ $self->{mac} }[ ( $len / 2 ) .. ( $len - 1 ) ] ),
     );
 }
 
@@ -519,15 +602,18 @@ sub to_eui48 {
     # be slightly evil here, so that hashrefs and objects work
     if ( is_eui64($self) ) {
         if ( @{ $self->{mac} }[3] == 0xff
-            and ( @{ $self->{mac} }[4] == 0xff or @{ $self->{mac} }[4] == 0xfe )
-          )
+            and
+            ( @{ $self->{mac} }[4] == 0xff or @{ $self->{mac} }[4] == 0xfe ) )
         {
 
             # convert to eui-48
             $self->{mac} = [ @{ $self->{mac} }[ 0 .. 2, 5 .. 7 ] ];
         }
         else {
-            croak 'eui-64 address is not derived from an eui-48 address';
+            my $e = 'eui-64 address is not derived from an eui-48 address';
+            croak "$e\n" if $self->{_die};
+            $NetAddr::MAC::errstr = $e;
+            return;
         }
     }
 
@@ -552,7 +638,9 @@ sub to_eui64 {
             @{ $self->{mac} }[ 0 .. 2 ], 0xff,
             0xfe,                        @{ $self->{mac} }[ 3 .. 5 ]
         ];
+
     }
+    else { return }
 
     return 1;
 }
@@ -570,9 +658,15 @@ sub mac_is_eui48 {
     my $mac = shift;
     croak 'please use is_eui48'
       if ref $mac eq __PACKAGE__;
-    croak 'argument must be a string' if ref $mac;
+    if ( ref $mac ) {
+        my $e = 'argument must be a string';
+        croak "$e\n" if $NetAddr::MAC::die_on_error;
+        $NetAddr::MAC::errstr = $e;
+        return;
+    }
 
-    return is_eui48( { mac => _mac_to_integers($mac) } )
+    $mac = _mac_to_integers($mac) or return;
+    return is_eui48( { mac => $mac } )
 
 }
 
@@ -587,9 +681,15 @@ sub mac_is_eui64 {
     my $mac = shift;
     croak 'please use is_eui64'
       if ref $mac eq __PACKAGE__;
-    croak 'argument must be a string' if ref $mac;
+    if ( ref $mac ) {
+        my $e = 'argument must be a string';
+        croak "$e\n" if $NetAddr::MAC::die_on_error;
+        $NetAddr::MAC::errstr = $e;
+        return;
+    }
 
-    return is_eui64( { mac => _mac_to_integers($mac) } )
+    $mac = _mac_to_integers($mac) or return;
+    return is_eui64( { mac => $mac } )
 
 }
 
@@ -604,9 +704,15 @@ sub mac_is_multicast {
     my $mac = shift;
     croak 'please use is_multicast'
       if ref $mac eq __PACKAGE__;
-    croak 'argument must be a string' if ref $mac;
+    if ( ref $mac ) {
+        my $e = 'argument must be a string';
+        croak "$e\n" if $NetAddr::MAC::die_on_error;
+        $NetAddr::MAC::errstr = $e;
+        return;
+    }
 
-    return is_multicast( { mac => _mac_to_integers($mac) } )
+    $mac = _mac_to_integers($mac) or return;
+    return is_multicast( { mac => $mac } )
 
 }
 
@@ -621,9 +727,15 @@ sub mac_is_unicast {
     my $mac = shift;
     croak 'please use is_unicast'
       if ref $mac eq __PACKAGE__;
-    croak 'argument must be a string' if ref $mac;
+    if ( ref $mac ) {
+        my $e = 'argument must be a string';
+        croak "$e\n" if $NetAddr::MAC::die_on_error;
+        $NetAddr::MAC::errstr = $e;
+        return;
+    }
 
-    return is_unicast( { mac => _mac_to_integers($mac) } )
+    $mac = _mac_to_integers($mac) or return;
+    return is_unicast( { mac => $mac } )
 
 }
 
@@ -638,9 +750,15 @@ sub mac_is_local {
     my $mac = shift;
     croak 'please use is_local'
       if ref $mac eq __PACKAGE__;
-    croak 'argument must be a string' if ref $mac;
+    if ( ref $mac ) {
+        my $e = 'argument must be a string';
+        croak "$e\n" if $NetAddr::MAC::die_on_error;
+        $NetAddr::MAC::errstr = $e;
+        return;
+    }
 
-    return is_local( { mac => _mac_to_integers($mac) } )
+    $mac = _mac_to_integers($mac) or return;
+    return is_local( { mac => $mac } )
 
 }
 
@@ -655,9 +773,15 @@ sub mac_is_universal {
     my $mac = shift;
     croak 'please use is_universal'
       if ref $mac eq __PACKAGE__;
-    croak 'argument must be a string' if ref $mac;
+    if ( ref $mac ) {
+        my $e = 'argument must be a string';
+        croak "$e\n" if $NetAddr::MAC::die_on_error;
+        $NetAddr::MAC::errstr = $e;
+        return;
+    }
 
-    return is_universal( { mac => _mac_to_integers($mac) } )
+    $mac = _mac_to_integers($mac) or return;
+    return is_universal( { mac => $mac } )
 
 }
 
@@ -676,9 +800,15 @@ sub mac_as_basic {
     my $mac = shift;
     croak 'please use as_basic'
       if ref $mac eq __PACKAGE__;
-    croak 'argument must be a string' if ref $mac;
+    if ( ref $mac ) {
+        my $e = 'argument must be a string';
+        croak "$e\n" if $NetAddr::MAC::die_on_error;
+        $NetAddr::MAC::errstr = $e;
+        return;
+    }
 
-    return as_basic( { mac => _mac_to_integers($mac) } )
+    $mac = _mac_to_integers($mac) or return;
+    return as_basic( { mac => $mac } )
 
 }
 
@@ -696,9 +826,15 @@ sub mac_as_bpr {
     my $mac = shift;
     croak 'please use as_basic'
       if ref $mac eq __PACKAGE__;
-    croak 'argument must be a string' if ref $mac;
+    if ( ref $mac ) {
+        my $e = 'argument must be a string';
+        croak "$e\n" if $NetAddr::MAC::die_on_error;
+        $NetAddr::MAC::errstr = $e;
+        return;
+    }
 
-    return as_bpr( { mac => _mac_to_integers($mac) } )
+    $mac = _mac_to_integers($mac) or return;
+    return as_bpr( { mac => $mac } )
 
 }
 
@@ -716,9 +852,15 @@ sub mac_as_cisco {
     my $mac = shift;
     croak 'please use as_cisco'
       if ref $mac eq __PACKAGE__;
-    croak 'argument must be a string' if ref $mac;
+    if ( ref $mac ) {
+        my $e = 'argument must be a string';
+        croak "$e\n" if $NetAddr::MAC::die_on_error;
+        $NetAddr::MAC::errstr = $e;
+        return;
+    }
 
-    return as_cisco( { mac => _mac_to_integers($mac) } )
+    $mac = _mac_to_integers($mac) or return;
+    return as_cisco( { mac => $mac } )
 
 }
 
@@ -736,9 +878,15 @@ sub mac_as_ieee {
     my $mac = shift;
     croak 'please use as_ieee'
       if ref $mac eq __PACKAGE__;
-    croak 'argument must be a string' if ref $mac;
+    if ( ref $mac ) {
+        my $e = 'argument must be a string';
+        croak "$e\n" if $NetAddr::MAC::die_on_error;
+        $NetAddr::MAC::errstr = $e;
+        return;
+    }
 
-    return as_ieee( { mac => _mac_to_integers($mac) } )
+    $mac = _mac_to_integers($mac) or return;
+    return as_ieee( { mac => $mac } )
 
 }
 
@@ -755,9 +903,15 @@ sub mac_as_ipv6_suffix {
     my $mac = shift;
     croak 'please use as_ipv6_suffix'
       if ref $mac eq __PACKAGE__;
-    croak 'argument must be a string' if ref $mac;
+    if ( ref $mac ) {
+        my $e = 'argument must be a string';
+        croak "$e\n" if $NetAddr::MAC::die_on_error;
+        $NetAddr::MAC::errstr = $e;
+        return;
+    }
 
-    return as_ipv6_suffix( { mac => _mac_to_integers($mac) } )
+    $mac = _mac_to_integers($mac) or return;
+    return as_ipv6_suffix( { mac => $mac } )
 
 }
 
@@ -776,9 +930,15 @@ sub mac_as_microsoft {
 
     croak 'please use as_microsoft'
       if ref $mac eq __PACKAGE__;
-    croak 'argument must be a string' if ref $mac;
+    if ( ref $mac ) {
+        my $e = 'argument must be a string';
+        croak "$e\n" if $NetAddr::MAC::die_on_error;
+        $NetAddr::MAC::errstr = $e;
+        return;
+    }
 
-    return as_microsoft( { mac => _mac_to_integers($mac) } )
+    $mac = _mac_to_integers($mac) or return;
+    return as_microsoft( { mac => $mac } )
 
 }
 
@@ -796,9 +956,15 @@ sub mac_as_singledash {
 
     croak 'please use as_singledash'
       if ref $mac eq __PACKAGE__;
-    croak 'argument must be a string' if ref $mac;
+    if ( ref $mac ) {
+        my $e = 'argument must be a string';
+        croak "$e\n" if $NetAddr::MAC::die_on_error;
+        $NetAddr::MAC::errstr = $e;
+        return;
+    }
 
-    return as_singledash( { mac => _mac_to_integers($mac) } )
+    $mac = _mac_to_integers($mac) or return;
+    return as_singledash( { mac => $mac } )
 
 }
 
@@ -817,9 +983,15 @@ sub mac_as_sun {
 
     croak 'please use as_sun'
       if ref $mac eq __PACKAGE__;
-    croak 'argument must be a string' if ref $mac;
+    if ( ref $mac ) {
+        my $e = 'argument must be a string';
+        croak "$e\n" if $NetAddr::MAC::die_on_error;
+        $NetAddr::MAC::errstr = $e;
+        return;
+    }
 
-    return as_sun( { mac => _mac_to_integers($mac) } )
+    $mac = _mac_to_integers($mac) or return;
+    return as_sun( { mac => $mac } )
 
 }
 
@@ -838,12 +1010,92 @@ sub mac_as_tokenring {
 
     croak 'please use as_tokenring'
       if ref $mac eq __PACKAGE__;
-    croak 'argument must be a string' if ref $mac;
+    if ( ref $mac ) {
+        my $e = 'argument must be a string';
+        croak "$e\n" if $NetAddr::MAC::die_on_error;
+        $NetAddr::MAC::errstr = $e;
+        return;
+    }
 
-    return as_tokenring( { mac => _mac_to_integers($mac) } )
+    $mac = _mac_to_integers($mac) or return;
+    return as_tokenring( { mac => $mac } )
 
 }
 
+=head1 ERROR HANDLING
+
+Prior to 0.8 every error resulted in a die (croak) which need to be caught.
+As I have used this module more, having to catch them all the time is tiresome.
+So from 0.8 onwards, errors result in an undef and something being set.
+
+For objects, this something is accessible via B<<$self->errstr>> otherwise
+ther error is in B<$NetAddr::MAC::errstr>;
+
+If you would like to have die (croak) instead, you can either set the global
+B<$NetAddr::MAC::die_on_error> or set the B<die_on_error> option when creating
+an object. When creating objects, the provided option takes priority over the
+global. So if you set the global, then all objects will die - unless you
+specifcy otherwise.
+
+=head2 Global examples
+
+Normal behaviour...
+
+  use NetAddr::MAC qw/mac_as_basic/;
+  $mac = mac_as_basic('aaaa.bbbb.cccc')
+      or die $NetAddr::MAC::errstr;
+
+If you want to catch exceptions (die/croak's)...
+
+  use NetAddr::MAC qw/mac_as_basic/;
+  $NetAddr::MAC::die_on_error = 1; # (or ++ if you like)
+
+  eval { # or use Try::Tiny etc.
+      $mac = mac_as_basic('aaaa.bbbb.cccc');
+  };
+  if ($@) {
+      # something bad happened, so handle it
+  }
+  # all good, so do something
+
+=head2 Object examples
+
+Normal behaviour...
+
+  use NetAddr::MAC;
+  my $obj = NetAddr::MAC->new( mac => 'aabbcc112233')
+      or die $NetAddr::MAC::errstr;
+
+  $mac = $obj->to_eui48
+      or dir $obj->errstr;
+
+If you want to catch exceptions (die/croak's)...
+
+  use NetAddr::MAC;
+  my $obj = NetAddr::MAC->new( mac => 'aabbcc112233', die_on_error => 1 );
+
+  eval { # or use Try::Tiny etc.
+      $mac = $obj->to_eui48
+  };
+  if ($@) {
+      # something bad happened, so handle it
+  }
+  # all good, so do something
+
+Or do it globally
+
+  use NetAddr::MAC;
+  $NetAddr::MAC::die_on_error = 1; # (or ++ if you like)
+  my $obj = NetAddr::MAC->new( mac => 'aabbcc112233');
+
+  eval { # or use Try::Tiny etc.
+      $mac = $obj->to_eui48
+  };
+  if ($@) {
+      # something bad happened, so handle it
+
+  }
+
 =head1 CREDITS
 
 Stolen lots of ideas and some pod content from L<Device::MAC> and L<Net::MAC>
@@ -851,18 +1103,20 @@ Stolen lots of ideas and some pod content from L<Device::MAC> and L<Net::MAC>
 =head1 TODO
 
  - moare tests!
+ - find bugs, squash them
+ - merge in your changes!
 
 =head1 SUPPORT
 
 Please use the RT system on CPAN to lodge bugs.
 
-Many young people like to use Github, by all means send me pull requests
+Many young people like to use Github, so by all means send me pull requests at
 
   https://github.com/djzort/NetAddr-MAC
 
 =head1 AUTHOR
 
-Dean Hamstead C<< <dean@fragfest.com.au> >>
+Dean Hamstead C<< <dean@bytefoundry.com.au> >>
 
 =head1 LICENSE
 
